#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>

#include "SNTP.h"

double unixtimetodouble( struct timeval* unixtime )
{
    return ( double ) unixtime->tv_sec + 1e-6 * ( double ) unixtime->tv_usec;
}

void printunixtime( const char* name, struct timeval* unixtime )
{
    time_t t;
    struct tm result;
    double dt;

    t = unixtime->tv_sec;
    localtime_r( &t, &result );

    dt = unixtimetodouble( unixtime );
    printf( "%s: %f s %s", name, dt ,asctime( &result ) );
}

int main( int argc, char** argv )
{
    int sockfd;
    struct sockaddr_in server;
    struct hostent *hp;

    if( argc != 2 ) {
        printf( "usage: %s host\n", argv[ 0 ] );
        exit( EXIT_FAILURE );
    }

    memset( &server, 0x0, sizeof( struct sockaddr_in ) );
    server.sin_family = AF_INET;
    server.sin_port = htons( 123 );

    hp = gethostbyname( argv[ 1 ] );
    if( hp == NULL) {
        printf("unknown server!\n");
        exit( EXIT_FAILURE );
    }

    memcpy( &server.sin_addr, hp->h_addr, sizeof( server.sin_addr ) );

    if( ( sockfd = socket( AF_INET, SOCK_DGRAM, 0) ) < 0 ) {
        perror("socket() error");
        exit( EXIT_FAILURE );
    }

    SNTPMessage sntpmsg;

    sntpmsg_clear( &sntpmsg );
    sntpmsg_set_li_vn_mode( &sntpmsg, SNTP_LI_NO_WARNING, SNTP_VN_IPV4ONLY, SNTP_MODE_CLIENT );

    struct timeval tval;
    gettimeofday( &tval, NULL );

    printunixtime( "Send", &tval );
    unixtime_to_sntptime( &sntpmsg.tx_timestamp, &tval );


    if( sendto( sockfd, &sntpmsg, sizeof( SNTPMessage ), 0, ( struct sockaddr*) &server, sizeof( struct sockaddr ) ) < 0) {
        perror("sendto() error");
        close( sockfd );
        exit( EXIT_FAILURE );
    }

    SNTPMessage reply;
    if( recvfrom( sockfd, &reply, sizeof( SNTPMessage), 0, NULL, NULL) < 0 ) {
        perror("recvfrom() error");
        close( sockfd );
        exit( EXIT_FAILURE );
    }


    struct timeval tv;
    sntptime_to_unixtime( &tv, &reply.ref_timestamp );
    printunixtime( "Reference", &tv );
    sntptime_to_unixtime( &tv, &reply.orig_timestamp );
    printunixtime( "Origin", &tv );
    sntptime_to_unixtime( &tv, &reply.rx_timestamp );
    printunixtime( "RX", &tv );
    sntptime_to_unixtime( &tv, &reply.tx_timestamp );
    printunixtime( "TX", &tv );

    close( sockfd );
    exit( EXIT_SUCCESS );
}
